@@ -6,9 +6,29 @@ use base qw( CatalystX::CRUD::Controller );
 use Carp;
 use Class::C3;
 
-__PACKAGE__->mk_accessors(qw( autocomplete_columns autocomplete_method ));
+#use Data::Dump qw( dump );
 
-our $VERSION = '0.003';
+__PACKAGE__->mk_accessors(
+    qw( autocomplete_columns autocomplete_method
+        hide_pk_columns fuzzy_search default_view
+        fmt_to_view_map
+        )
+);
+
+__PACKAGE__->config(
+    autocomplete_columns => [],
+    autocomplete_method  => undef,
+    hide_pk_columns      => 1,
+    fuzzy_search         => 0,
+    default_view         => 'YUI',
+    fmt_to_view_map      => {
+        'html' => 'YUI',
+        'json' => 'YUI',
+        'xls'  => 'Excel',
+    },
+);
+
+our $VERSION = '0.004';
 
 =head1 NAME
 
@@ -16,18 +36,113 @@ CatalystX::CRUD::YUI::Controller - base controller
 
 =head1 SYNOPSIS
 
- # TODO
+ package MyApp::Controller::Foo;
+ use strict;
+ 
+ # ISA order is important
+ use base qw(
+    CatalystX::CRUD::YUI::Controller
+    CatalystX::CRUD::Controller::RHTMLO
+ );
+ 
+ __PACKAGE__->config(
+    autocomplete_columns => [qw( foo bar )],
+    autocomplete_method  => 'foo',
+    hide_pk_columns      => 1,
+    fuzzy_search         => 0,
+    default_view         => 'YUI',
+    fmt_to_view_map      => {
+        'html' => 'YUI',
+        'json' => 'YUI',
+        'xls'  => 'Excel',
+    },
+ );
+ 
+ 1;
+ 
 
 =head1 DESCRIPTION
 
- # TODO
- 
+This is a base controller class for use with CatalystX::CRUD::YUI
+applications. It implements URI and internal methods that the
+accompanying .tt and .js files rely upon.
+
+=head1 CONFIGURATION
+
+See SYNOPSIS for config options specific to this class.
+
+=over
+
+=item autocomplete_columns
+
+See the autocomplete_columns() method. Value should be an array ref.
+
+=item autocomplete_method
+
+See the autocomplete_method() method. Value should be a method name.
+
+=item hide_pk_columns
+
+Used in the DataTable class. Boolean setting indicating whether the 
+primary key column(s) should appear in the YUI DataTable listings
+or not. Default is true.
+
+=item fuzzy_search
+
+If true, the C<cxc-fuzzy> param will be appended to all search queries
+via the .tt files. See CatalystX::CRUD::Model::Utils for more
+documentation about the C<cxc-fuzzy> param.
+
+=item default_view
+
+The name of the View to use if C<cxc-fmt> is not specified. This
+should be the name of a View that inherits from CatalystX::CRUD::YUI::View.
+The default is 'YUI'.
+
+=item fmt_to_view_map
+
+Hash ref of C<cxc-fmt> types to View names. Used in end() to determine
+which View to set.
+
+=back
+  
 =head1 METHODS
 
 Only new or overridden method are documented here.
 
 =cut
 
+=head2 new
+
+Overrides base method just to call next::method and ensures
+config() gets merged correctly.
+
+=cut
+
+sub new {
+    my ( $class, $app_class, $args ) = @_;
+
+    my $self = $class->next::method( $app_class, $args );
+
+    if ( $self->isa('CatalystX::CRUD::REST') ) {
+
+        # must merge hashes manually due to inheritance bug with C::D::I
+        # this only seems to happen when inheriting also from CXC::REST
+        # either because the inheritance chain is too deep or something
+        # else mysterious...
+        $self->config(
+            $self->merge_config_hashes( $self->_config, __PACKAGE__->config )
+        );
+        for my $key ( keys %{ $self->config } ) {
+            next unless $self->can($key);
+            next if defined $self->$key;
+            $self->$key( $self->config->{$key} );
+        }
+
+    }
+    return $self;
+}
+
 =head2 json_mime
 
 Returns JSON MIME type. Default is 'application/json; charset=utf-8'.
@@ -48,8 +163,11 @@ sub auto : Private {
     my ( $self, $c, @arg ) = @_;
 
     # in YUI > 2.5.0 the paginator uses non-sql-friendly sort dir values
-    if ( exists $c->req->params->{_dir} ) {
-        $c->req->params->{_dir} =~ s/yui\-dt\-//;
+    for my $param_name (qw( cxc-dir _dir )) {
+        if ( exists $c->req->params->{$param_name} ) {
+            $c->req->params->{$param_name} =~ s/yui\-dt\-//;
+            last;
+        }
     }
 
     $self->next::method( $c, @arg );
@@ -275,10 +393,36 @@ sub form_to_object {
     my ( $self, $c ) = @_;
 
     #carp "form_to_object";
+
+    # check for o2m via datatable-click to avoid
+    # re-saving duplicates
+    if ( $c->req->params->{'cxc-o2m'} ) {
+        my $obj      = $c->stash->{object};
+        my $match    = 0;
+        my $eligible = 0;
+        for my $param ( keys %{ $c->req->params } ) {
+            if ( $obj->can($param) ) {
+                $eligible++;
+                if ( $obj->$param eq $c->req->params->{$param} ) {
+                    $match++;
+                    $c->log->debug( "object already has $param = "
+                            . $c->req->params->{$param} )
+                        if $c->debug;
+                }
+            }
+        }
+        if ( $match == $eligible ) {
+            $c->response->body('Object already related');
+            $c->response->status(400);
+            return;
+        }
+    }
+
     my $obj = $self->next::method($c);
+
     if (   !$obj
-        && exists $c->req->params->{return}
-        && $c->req->params->{return} eq 'json' )
+        && exists $c->req->params->{'cxc-fmt'}
+        && $c->req->params->{'cxc-fmt'} eq 'json' )
     {
         $c->response->status(500);
         my $err = $self->all_form_errors( $c->stash->{form} );
@@ -290,40 +434,26 @@ sub form_to_object {
 
 =head2 postcommit
 
-Overrides base method to render response as JSON where necessary.
-The C<return> request param is checked for the string 'json'
-and the object is serialized accordingly.
+Overrides base method to re-read object from db.
 
 =cut
 
 sub postcommit {
     my ( $self, $c, $obj ) = @_;
 
+    $c->log->debug("postcommit YUI") if $c->debug;
+
     # get whatever auto-set values were set.
     unless ( $c->action->name eq 'rm' ) {
-        $obj->read if $obj->can('read');
-    }
-
-    if ( exists $c->req->params->{return} ) {
-
-        my $type = $c->req->params->{return};
-        if ( $type eq 'json' ) {
-
-            $c->log->debug("JSONifying object for response") if $c->debug;
-
-            $c->stash( object   => $obj );               # is this necessary?
-            $c->stash( template => 'crud/jsonify.tt' )
-                unless defined $c->stash->{template};
-            $c->response->content_type( $self->json_mime );
-
+        if ( $self->model_adapter ) {
+            $self->model_adapter->read( $c, $obj );
         }
         else {
-            $self->throw_error("unknown return type: $type");
+            $obj->read;
         }
     }
-    else {
-        $self->next::method( $c, $obj );
-    }
+
+    $self->next::method( $c, $obj );
 
     return $obj;
 }
@@ -341,7 +471,7 @@ Set this value in config(). Default is a no-op.
 # it is marked with the _private prefix for now.
 sub _get_autocomplete_columns {
     my ( $self, $c ) = @_;
-    return $self->autocomplete_columns || [];
+    return $self->autocomplete_columns;
 }
 
 =head2 autocomplete_method
@@ -400,23 +530,82 @@ sub autocomplete : Local {
     #warn "ac_columns: " . dump $ac_columns;
     #warn "ac_method: " . $ac_method;
 
-    $p->{_fuzzy}     = 1;
-    $p->{_page_size} = $p->{l};
-    $p->{_op}        = 'OR';
+    $p->{'cxc-fuzzy'}     = 1;
+    $p->{'cxc-page_size'} = $p->{l};
+    $p->{'cxc-op'}        = 'OR';
+    $p->{'cxc-fmt'}       = 'json';
     $p->{$_} = $p->{query} for @$ac_columns;
-    my $query = $c->model( $self->model_name )->make_query($ac_columns);
+    my $query = $self->do_model( $c, 'make_query', $ac_columns );
 
-    $c->stash->{results} = $c->model( $self->model_name )->search(
-        query   => $query->{query},
-        sort_by => $query->{sort_by},
-        limit   => $query->{limit},
-    );
+    $c->stash->{results}    = $self->do_model( $c, 'search', $query );
     $c->stash->{ac_field}   = $p->{c};
     $c->stash->{ac_method}  = $ac_method;
     $c->stash->{ac_columns} = $ac_columns;
     $c->stash->{template}   = 'crud/autocomplete.tt';
 }
 
+=head2 end
+
+Uses the RenderView ActionClass.
+
+Prior to passing to view, sets C<current_view> based
+on the C<cxc-fmt> request parameter, defaulting
+to 'YUI'.
+
+B<NOTE:>This assumes you have a View class called
+C<YUI> that inherits from CatalystX::CRUD::YUI::View.
+
+=cut
+
+sub end : ActionClass('RenderView') {
+    my ( $self, $c, @arg ) = @_;
+    my $fmt = $c->req->params->{'cxc-fmt'} || 'html';
+
+    if ($fmt) {
+        $c->log->debug("fmt = $fmt") if $c->debug;
+        my $view_class = $self->fmt_to_view_map->{$fmt}
+            || $self->default_view;
+        $c->stash->{current_view} ||= $view_class;
+
+        # special case for add() and save()
+        if ( $fmt eq 'json' and defined $c->stash->{object} ) {
+
+            # catch errors and abort so we don't return json
+            if ( $self->has_errors($c) ) {
+                my @err = @{ $c->error };
+                $c->log->error($_) for @err;
+                $c->clear_errors;
+                $c->response->body('Server error');
+                $c->response->status(500);
+                return;
+            }
+
+            $c->log->debug("JSONifying object for response") if $c->debug;
+            $c->stash( template => 'crud/jsonify.tt' )
+                unless defined $c->stash->{template};
+            $c->stash(
+                serial_args => {
+                    object => $c->stash->{object},
+                    takes_object_as_argument =>
+                        $self->form($c)->metadata->takes_object_as_argument,
+                    col_names => $self->form($c)->metadata->field_methods,
+                }
+            );
+
+            unless ( defined $c->res->status and $c->res->status =~ m/^[45]/ )
+            {
+                $c->res->content_type( $self->json_mime );
+
+                # in case postcommit() set these
+                $c->res->location('');
+                $c->res->status(200);
+            }
+        }
+
+        $c->log->debug("view set to $view_class") if $c->debug;
+    }
+}
+
 1;
 
 __END__
@@ -440,7 +629,7 @@ sponsored the development of this software.
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2008 by the Regents of the University of Minnesota.
-All Rights Reserved.
+
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
