@@ -128,11 +128,16 @@ YAHOO.crud.add_matrix_row = function( matrix ) {
                 }
                 var pk = pk_vals.join(';;');
        
-                //alert(matrix.opts.name + ": got pk " + pk + ' cmap: ' + matrix.opts.cmap.toJSONString());
-                if (matrix.opts.cmap) {
-                    // just need to update the foreign key value in selected row
-                    var postData = matrix.opts.cmap[1] + "=" + matrix.opts.parent_oid;
-                    var url = matrix.opts.info_url + '/' + pk + '/save?return=json';
+                //YAHOO.crud.log(matrix.opts.name + ": got pk " + pk + matrix.opts.foreign.column_map);
+                if (matrix.opts.foreign.column_map) {
+                    // just need to update the foreign key value(s) in selected row
+                    var postData = '';
+                    for(i=1; i<matrix.opts.foreign.column_map.length; i+=2) {
+                        postData += matrix.opts.foreign.column_map[i] + 
+                                    "=" + matrix.opts.parent.column_map[i] + '&';
+                    }
+                    var url = matrix.opts.foreign.url + '/' + 
+                                pk + '/save?cxc-fmt=json&cxc-o2m=1';
                     //alert("POST url: " + url + '?' + postData);
                     
                     var req = YAHOO.util.Connect.asyncRequest('POST', url,
@@ -141,7 +146,8 @@ YAHOO.crud.add_matrix_row = function( matrix ) {
                                 if (o.responseText !== undefined) {
                                     var newRow = o.responseText.parseJSON();
                                     matrix.myDataTable.addRow(newRow, 0);
-                                    YAHOO.util.Dom.get('panel_msg').innerHTML = 'Record added';
+                                    YAHOO.util.Dom.get('panel_msg').innerHTML = 
+                                        'Record added';
                                 }
                                 else {
                                     alert("unknown server error");
@@ -157,7 +163,8 @@ YAHOO.crud.add_matrix_row = function( matrix ) {
                     
                 }
                 else {
-                    var url = matrix.opts.parent_url + '/' + matrix.opts.parent + '/' + pk + '/add';
+                    var url = matrix.opts.parent.url + '/' + matrix.opts.foreign.name + 
+                        '/' + pk + '/add';
                     //alert("add_m2m :" + url);
                     
                     var req = YAHOO.util.Connect.asyncRequest('POST', url,
@@ -167,7 +174,8 @@ YAHOO.crud.add_matrix_row = function( matrix ) {
                                     var newRow = o.responseText.parseJSON();
                                     newRow._remove = 'X';
                                     matrix.myDataTable.addRow(newRow, 0);
-                                    YAHOO.util.Dom.get('panel_msg').innerHTML = 'Record added';
+                                    YAHOO.util.Dom.get('panel_msg').innerHTML = 
+                                        'Record added';
                                     YAHOO.crud.decorateRemoveCells();
                                 }
                                 else {
@@ -206,7 +214,11 @@ YAHOO.crud.add_matrix_row = function( matrix ) {
         success: handleSuccess, 
         failure: YAHOO.crud.handleXHRFailure
     };
-    var request = YAHOO.util.Connect.asyncRequest('GET', matrix.opts.count_url, callback);
+    var request = YAHOO.util.Connect.asyncRequest(
+        'GET', 
+        matrix.opts.parent.count, 
+        callback
+    );
     
 }
 
@@ -235,7 +247,7 @@ YAHOO.crud.panelled_related_records_matrix = function( matrixOpts ) {
         Dom        = YAHOO.util.Dom,
         Event      = YAHOO.util.Event;
 
-    var mySource = new DataSource(matrixOpts.panel_url);
+    var mySource = new DataSource(matrixOpts.foreign.panel);
     mySource.responseType   = DataSource.TYPE_JSON;
     mySource.responseSchema = {
         resultsList : 'records',
@@ -254,7 +266,7 @@ YAHOO.crud.panelled_related_records_matrix = function( matrixOpts ) {
                 for(i=0; i<matrixOpts.colFilter.length; i++) {
                     req += '&' + matrixOpts.colFilter[i] + '=' + query;
                 }
-                req += '&_op=OR&_fuzzy=1';
+                req += '&cxc-op=OR&cxc-fuzzy=1';
             }
             // remember this query in state, from which buildQueryString() will work.
             YAHOO.crud.panel_state.filter = req;
@@ -290,7 +302,8 @@ YAHOO.crud.panelled_related_records_matrix = function( matrixOpts ) {
         var ACF = new YAHOO.widget.DS_JSFunction(getFilter);
         ACF.minQueryLength = 0;
         ACF.maxCacheEntries = 0; // always send request
-        var ACFilter = new YAHOO.widget.AutoComplete("panel_ac", "panel_ac_hidden", ACF);
+        var ACFilter = new YAHOO.widget.AutoComplete(
+                        "panel_ac", "panel_ac_hidden", ACF);
         ACFilter.textboxKeyEvent.subscribe(checkFilterKey);
     }
     else {
@@ -333,7 +346,8 @@ YAHOO.crud.panelled_related_records_matrix = function( matrixOpts ) {
                     "desc" : "asc";
         }
         
-        // must always return to page 1 because we can't rely on how sorted results are paged.
+        // must always return to page 1 because we can't rely 
+        // on how sorted results are paged.
         YAHOO.crud.panel_state.startIndex = 0;
         YAHOO.crud.panel_state.dir = sDir;
         YAHOO.crud.panel_state.sort = oColumn.key;
@@ -431,7 +445,7 @@ YAHOO.crud.related_records_matrix = function( opts ) {
         myFields[i] = opts.fields[i];
     }
     
-    if (opts.add_remove_button) {
+    if (opts.foreign.rm_button) {
         myColumnDefs.push(
             {
                 key:"_remove", 
@@ -444,7 +458,7 @@ YAHOO.crud.related_records_matrix = function( opts ) {
 
     // create handler for rowclick. delete a M2M or goto related record.
     var rowClickHandler;
-    if ( opts.m2m ) {
+    if ( opts.foreign.m2m ) {
       rowClickHandler = function(oArgs) {
         // get pk value for this row
         // 'this' is DataTable object
@@ -468,7 +482,7 @@ YAHOO.crud.related_records_matrix = function( opts ) {
                 // make ajax call to remove relationship
                 YAHOO.util.Connect.asyncRequest(
                 'POST',
-                opts.rm_m2m_url + pk + '/remove',
+                opts.parent.url + '/' + opts.foreign.name + '/' + pk + '/remove',
                 {
                     success: function (o) {
                         if (o.responseText == 'Ok') {
@@ -503,18 +517,18 @@ YAHOO.crud.related_records_matrix = function( opts ) {
                 );
             }
         }
-        else if (opts.no_follow) {
+        else if (opts.foreign.view_perm) {
             // do nothing
         
         }
         // redirect to detail screen
         else {
-            var newurl      = opts.info_url + '/' + pk + '/' + opts.row_url_method;
+            var newurl      = opts.foreign.url + '/' + pk + '/' + opts.foreign.method;
             window.location.href = newurl;
         }
       };
     }
-    else if (opts.no_follow) {
+    else if (opts.foreign.view_perm) {
     
       rowClickHandler = function(oArgs) {
         // do nothing.
@@ -537,26 +551,28 @@ YAHOO.crud.related_records_matrix = function( opts ) {
         for(i=0; i<pks.length; i++) {
             pk_vals.push( encodeURIComponent( record.getData(pks[i]) ) );
         }
-        var pk = pk_vals.join(';;');
-        var newurl      = opts.info_url + '/' + pk + '/' + opts.row_url_method;
+        var pk          = pk_vals.join(';;');
+        var newurl      = opts.foreign.url + '/' + pk + '/' + opts.foreign.method;
         window.location.href = newurl;   
       };
     }
     
     var Matrix = YAHOO.crud.create_results_matrix(
     {
-        colDefs:    myColumnDefs,
-        fields:     myFields,
-        url:        opts.url,  
-        anchor:     opts.anchor,
-        pageSize:   opts.pageSize,
-        pagerId:    opts.pagerId,
-        pk:         opts.pk,
-        sortBy:     opts.sortBy,
-        totalPages: opts.totalPages,
-        totalResults: opts.totalResults,
-        divId:      opts.divId,
-        rowClickHandler: rowClickHandler
+        colDefs:        myColumnDefs,
+        fields:         myFields,
+        anchor:         opts.anchor,
+        pageSize:       opts.pageSize,
+        pagerId:        opts.pagerId,
+        pk:             opts.pk,
+        sortBy:         opts.sortBy,
+        totalPages:     opts.totalPages,
+        totalResults:   opts.totalResults,
+        divId:          opts.divId,
+        rowClickHandler: rowClickHandler,
+        hidePKColumns:  opts.hidePKColumns,
+        parent:         opts.parent,
+        foreign:        opts.foreign
     }
     );
 
@@ -589,19 +605,19 @@ YAHOO.crud.decorateRemoveCells = function() {
 // method to generate a query string for the DataSource.  
 // Also used as the state indicator for the History Manager
 YAHOO.crud.generateStateString = function (start,key,dir,psize) {
-    return  "&_page_size="  +   psize   + 
-            "&_offset="     +   start   +
-            "&_sort="       +   key     +
-            "&_dir="        +   dir;
+    return  "&cxc-page_size="  +   psize   + 
+            "&cxc-offset="     +   start   +
+            "&cxc-sort="       +   key     +
+            "&cxc-dir="        +   dir;
 };
 
 // method to extract the key values from the state string
 YAHOO.crud.parseStateString = function (state) {
     return {
-        results    : /\b_page_size=(\d+)/.test(state)   ? parseInt(RegExp.$1) : 20,
-        startIndex : /\b_offset=(\d+)/.test(state)      ? parseInt(RegExp.$1) : 0,
-        sort       : /\b_sort=(\w+)/.test(state)        ? RegExp.$1 : 'id',
-        dir        : /\b_dir=([\w\-]+)/.test(state)     ? RegExp.$1 : 'asc'
+        results    : /\bcxc-page_size=(\d+)/.test(state)   ? parseInt(RegExp.$1) : 20,
+        startIndex : /\bcxc-offset=(\d+)/.test(state)      ? parseInt(RegExp.$1) : 0,
+        sort       : /\bcxc-sort=(\w+)/.test(state)        ? RegExp.$1 : 'id',
+        dir        : /\bcxc-dir=([\w\-]+)/.test(state)     ? RegExp.$1 : 'asc'
     }
 };
 
@@ -662,13 +678,13 @@ YAHOO.crud.create_results_matrix = function( matrixOpts ) {
         myDataTable,
         myPaginator;
     
-    YAHOO.crud.log("MyMatrix called");
-    YAHOO.crud.log(matrixOpts);
+    //YAHOO.crud.log("MyMatrix called");
+    //YAHOO.crud.log(matrixOpts);
         
     // function used to intercept pagination requests
     var handlePagination = function (state,datatable) {
     
-        YAHOO.crud.log(state);
+        //YAHOO.crud.log(state);
     
         var sortedBy  = datatable.get('sortedBy');
 
@@ -722,7 +738,7 @@ YAHOO.crud.create_results_matrix = function( matrixOpts ) {
     YAHOO.crud.historyList.push(
       function() {
       
-        YAHOO.crud.log("onReady for History " + matrixOpts.anchor);
+        //YAHOO.crud.log("onReady for History " + matrixOpts.anchor);
         
         // Pull the state from the History Manager, or default from the
         // initial state.  Parse the state string into an object literal.
@@ -731,7 +747,7 @@ YAHOO.crud.create_results_matrix = function( matrixOpts ) {
             state          = YAHOO.crud.parseStateString(initialRequest);
 
         // Create the DataSource
-        myDataSource = new YAHOO.util.DataSource(matrixOpts.url);
+        myDataSource = new YAHOO.util.DataSource(matrixOpts.parent.datatable);
         myDataSource.responseType = YAHOO.util.DataSource.TYPE_JSON;
         myDataSource.responseSchema = {
             resultsList:    "records",
@@ -739,7 +755,7 @@ YAHOO.crud.create_results_matrix = function( matrixOpts ) {
             fields:         matrixOpts.fields
         };
         
-        YAHOO.crud.log("this = ", this);
+        //YAHOO.crud.log("this = ", this);
         
         // Column definitions
         var myColumnDefs = matrixOpts.colDefs;
@@ -781,9 +797,11 @@ YAHOO.crud.create_results_matrix = function( matrixOpts ) {
         );
         
         // remember these for callbacks
-        myMatrix.myPaginator = myPaginator;
+        myMatrix.myPaginator  = myPaginator;
         myMatrix.myDataSource = myDataSource;
-        myMatrix.myDataTable = myDataTable;
+        myMatrix.myDataTable  = myDataTable;
+        
+        //YAHOO.crud.log("callbacks set: " + matrixOpts.anchor);
 
         // Listen to header link clicks to sort the column
         myDataTable.subscribe('theadCellClickEvent', myDataTable.onEventSortColumn);
@@ -798,9 +816,35 @@ YAHOO.crud.create_results_matrix = function( matrixOpts ) {
         // Enables single-mode row selection
         myDataTable.set("selectionMode","single");
         
+        // optionally hide the PK column 
+        // (esp nice for auto-increment values which are meaningless to user)
+        if (matrixOpts.hidePKColumns) {
+        
+            //YAHOO.crud.log(matrixOpts.anchor + " hidePKColumns == true");
+            
+            // find each column that is a PK
+            for (     var i = 0; i<matrixOpts.fields.length; i++ ) {
+                for ( var j = 0; j<matrixOpts.pk.length;     j++ ) {
+                
+                    var colName = matrixOpts.fields[i];
+                    var pkName  = matrixOpts.pk[j];
+                    //YAHOO.crud.log("compare colname " + colName + " == pkName " + pkName);
+                
+                    if ( colName == pkName ) {
+                        var oColumn = myDataTable.getColumn(i);
+                        myDataTable.hideColumn(oColumn);
+                    }
+                }
+            }
+            
+        }
+        
+        //YAHOO.crud.log("hidePKColumns set: " + matrixOpts.anchor);
+
+        
         // make each row click-able with action defined by caller.
         // Subscribe to events for row selection
-        if(!matrixOpts.no_follow) {
+        if(!matrixOpts.foreign.view_perm) {
             myDataTable.subscribe("rowMouseoverEvent", myDataTable.onEventHighlightRow);
             myDataTable.subscribe("rowMouseoutEvent",  myDataTable.onEventUnhighlightRow);
         }
@@ -814,6 +858,8 @@ YAHOO.crud.create_results_matrix = function( matrixOpts ) {
         
         // set event listeners on paginator page nums to create hover effect
         YAHOO.crud.hover_class_on_mousemove(matrixOpts.pagerId);
+        
+        //YAHOO.crud.log("handlers set: " + matrixOpts.anchor);
 
         // set up autocomplete filter
         var buildQueryString = function (state, datatable) {
@@ -841,7 +887,7 @@ YAHOO.crud.create_results_matrix = function( matrixOpts ) {
                     for(i=0; i<matrixOpts.colFilter.length; i++) {
                         req += '&' + matrixOpts.colFilter[i] + '=' + query;
                     }
-                    req += '&_op=OR&_fuzzy=1';
+                    req += '&cxc-op=OR&cxc-fuzzy=1';
                 }
                 myDataSource.sendRequest(req, myDataTable.onDataReturnInitializeTable, myDataTable);
             };
@@ -859,7 +905,10 @@ YAHOO.crud.create_results_matrix = function( matrixOpts ) {
             var ACFilter = new YAHOO.widget.AutoComplete("results_ac", "results_ac_hidden", ACF);
             ACFilter.textboxKeyEvent.subscribe(checkFilterKey);
         }
-        
+                
+                
+        //YAHOO.crud.log("init of create_datatable complete: " + matrixOpts.anchor);
+                
       } // end function()
     
     );  // end .push
@@ -870,7 +919,7 @@ YAHOO.crud.create_results_matrix = function( matrixOpts ) {
 
 YAHOO.crud.init_histories = function () {
 
-    YAHOO.crud.log("HistoryList init " + YAHOO.crud.historyList.length);
+    //YAHOO.crud.log("HistoryList init " + YAHOO.crud.historyList.length);
 
     // set an onReady function that calls each function in our list
     YAHOO.util.History.onReady(function() {
